#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "pixel.h"
#include "font.h"
#include "mags.h"

#define DRIVEPINS (_BV(PB0) | _BV(PB1) | _BV(PB2))
#define SWITCH _BV(PB4)

#define ROUNDS 10	// times to charlieplex LEDS per column

void init() {
	cli();
	
	// switch input causes interrupt
	DDRB &= ~SWITCH;
	PORTB |= SWITCH;
	PCMSK = SWITCH;
	GIMSK = _BV(PCIF);

	// sleep mode
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);
	
	sei();
}

void charlieplex(unsigned char pixels) {
	int i,j;

	// we do ROUNDS of charlieplexing
	for(j = 0; j < ROUNDS; j++) {
		for(i = 0; i < 7; i++) {
			unsigned char data = pgm_read_byte(&pixelData[i]);
						
			// this is an attempt to take constant time regardless
			// of pixel state, ensuring equal brightness per pixel
			DDRB = (DDRB & ~DRIVEPINS) | ((data >> 4) & ((pixels & (1 << i))?(0x07):(0)));
			PORTB = (PORTB & ~DRIVEPINS) | (data & ((pixels & (1 << i))?(0x07):(0)));
		}
	}
	PORTB &= ~DRIVEPINS;
	DDRB &= ~DRIVEPINS;
}

void displayDigit(int n, char direction) {
	int i;
	for(i = 0; i < 4; i++) {
		int idx = (direction)?((4*n) + i):((4*n) + 3 - i);
		charlieplex(pgm_read_byte(&fontData[idx]));
	}
	// 1 column gap
	charlieplex(0);
}

unsigned int count = 0;

void display() {
	unsigned char i, n, first = 1;
	unsigned m, c = count;

	for(i = 0; i < 5; i++) {
		m = pgm_read_word(&mags[(unsigned)i]);
		for(n = 0; c >= m; c -= m) n++;
		if(first && n < 1) continue;
		first = 0;
		displayDigit(n, 1);
	}

	// zero
	if(first) displayDigit(0, 1);
}

ISR(PCINT0_vect) {
	if(!(PINB & SWITCH)) {
		_delay_ms(50);
		display();
		count++;
		_delay_ms(50);
	}
}

int main() {

	init();	
	while(1) sleep_mode();

	return 0; // unreached
}
